// ------------------------------------------------------------
// Tetrahedron env map
// 
// Copyright (c) 2004 IMAGIRE Takashi. All rights reserved.
// ------------------------------------------------------------
#include "paraboloid.fx"

// ------------------------------------------------------------
// Global variables(O[oϐ)
// ------------------------------------------------------------
float4x4 mWVP;		// Local-Projection matrix
float4x4 mWV;		// Local-View matrix
float4x4 mW;		// Local-World matrix
float4 vEye;		// Eye position(Local coordinate)

float MAP_WIDTH;	// texture width
float MAP_HEIGHT;	// texture height

// ------------------------------------------------------------
// Textures(eNX`)
// ------------------------------------------------------------
texture SrcTex;
sampler SrcSamp = sampler_state
{
    Texture = <SrcTex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
// ------------------------------------------------------------
texture DecaleTex;
sampler DecaleSamp = sampler_state
{
    Texture = <DecaleTex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
// ------------------------------------------------------------
// Dual paraboloid environment maps(o}bv)
// ------------------------------------------------------------
texture ParaboloidFrontTex;
sampler ParaboloidFrontSamp = sampler_state
{
    Texture = <ParaboloidFrontTex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
// ------------------------------------------------------------
texture ParaboloidBackTex;
sampler ParaboloidBackSamp = sampler_state
{
    Texture = <ParaboloidBackTex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};


// ------------------------------------------------------------
// Vertex shader output(_VF[_sNZVF[_ɓnf[^)
// ------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Pos			: POSITION;
    float4 Color		: COLOR0;
	float2 Tex0			: TEXCOORD0;
	float2 Tex1			: TEXCOORD1;
	float3 Normal		: TEXCOORD2;
	float2 DecaleTex	: TEXCOORD3;
};

// ------------------------------------------------------------
// Simple vertex shader program(wi`撸_VF[_vO)
// ------------------------------------------------------------
VS_OUTPUT VS (
      float4 Pos    : POSITION           // (f̒_)
     ,float4 Normal : NORMAL             // (@xNg)
     ,float4 Tex0   : TEXCOORD0	         // (eNX`W)
){
    VS_OUTPUT Out = (VS_OUTPUT)0;
    
	// Position(ʒuW)
	Out.Pos = mul( Pos, mWVP );
	
	// Texture coordinate(eNX`W)
    Out.Tex0 = Tex0;
	
    return Out;
}




// ------------------------------------------------------------
// Simple pixel shader program(wi`sNZVF[_vO)
// ------------------------------------------------------------
float4 PS (VS_OUTPUT In) : COLOR
{
	float4 Out;
	
	// Color(F)
	Out = tex2D( SrcSamp, In.Tex0 );
	
	return Out;
}





// ------------------------------------------------------------
// Parabploid vertex shader program(ϊ_VF[_vO)
// ------------------------------------------------------------
VS_OUTPUT VS_Parabploid (
      float4 Pos    : POSITION           // (f̒_)
){
    VS_OUTPUT Out = (VS_OUTPUT)0;
    
	// Position(ʒuW)
	Out.Pos = paraboloid( Pos, mWV );
	
	// Texture coordinate(eNX`W)
    Out.Tex0.x = + 0.5 * Pos.x + 0.5;
    Out.Tex0.y = - 0.5 * Pos.y + 0.5;
	
    return Out;
}


// ------------------------------------------------------------
// Dual-Parabploid environment mapping vertex shader program(}bv_VF[_vO)
// ------------------------------------------------------------
VS_OUTPUT VS_Envmap (
      float4 Pos    : POSITION           // (f̒_)
    , float4 Normal : NORMAL             // (@xNg)
    , float2 Tex    : TEXCOORD0          // (eNX`W)
      )
{
    VS_OUTPUT Out = (VS_OUTPUT)0;
    float3 ray;
    float3 ref;
    
	// Position(ʒuW)
	Out.Pos = mul( Pos, mWVP );
	
	// Color(F)
	Out.Color = 0.0f;
	
	// Texture Coordinate(eNX`W)
    ray = normalize( Pos - vEye ).xyz;
	ref = reflect( ray, Normal.xyz );		// Reflection vector(˃xNg)
	ref = mul( ref, mW );

	Out.Tex0.x = 0.5 * ( 1 + ref.x/(1+ref.z));
	Out.Tex0.y = 0.5 * ( 1 - ref.y/(1+ref.z));
	Out.Tex1.x = 0.5 * ( 1 - ref.x/(1-ref.z));
	Out.Tex1.y = 0.5 * ( 1 - ref.y/(1-ref.z));
	
	Out.Color.w = ref.z + 0.5f;		// The judgement of the signature of the z-component of the reflection vector(˃xNg̐̔)
	
	// Normal vector in the view coordinate system(r[Wnł̖@xNg)
	Out.Normal = mul( Normal.xyz, mWV );
	
	Out.DecaleTex = Tex;
	
    return Out;
}


// ------------------------------------------------------------
// Sampling environment map(}bv̓ǂݍ)
// ------------------------------------------------------------
float4 GetEnvMap( float2 tex_front, float2 tex_back, float NE, bool front_or_back )
{
	float4 env;
	
	if( front_or_back ){
		env = tex2D( ParaboloidFrontSamp, tex_front );
	}else{
		env = tex2D( ParaboloidBackSamp,  tex_back  );
	}
	
	// Fresnel term (tl)
	float F = 0.1 + 1.0 * pow(1+NE, 4);
	
	return F * env;
}

// ------------------------------------------------------------
// Full Global Illumination(S݃sNZVF[_vO)
// ------------------------------------------------------------
float4 PS_Envmap (VS_OUTPUT In) : COLOR
{
	float4 Env = GetEnvMap( In.Tex0, In.Tex1, In.Normal.z, 0.5 < In.Color.w );// Environment map(}bv)
	float4 decale = tex2D( DecaleSamp, In.DecaleTex );
	
	return 0.6 * decale + Env;
}






// ------------------------------------------------------------
// eNjbN
// ------------------------------------------------------------
technique TShader
{
    pass P0 // Simple transform(n``)
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_1_1 PS();
    }
    
    pass P1	// Paraboloid transform(ϊn``)
    {
        VertexShader = compile vs_1_1 VS_Parabploid();
        PixelShader  = compile ps_1_1 PS();
    }
    
    pass P2	// }bv𒣂
    {
        VertexShader = compile vs_1_1 VS_Envmap();
        PixelShader  = compile ps_2_0 PS_Envmap();
    }
}
